package sp_upg_2;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Shape;
import java.awt.font.TextLayout;
import java.awt.geom.AffineTransform;
import java.awt.geom.Area;
import java.awt.geom.Path2D;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.util.Arrays;

public class Kraj {

    public static int pocet = 0;
    public static Shape[] kraje = new Shape[14];
    String nazev;
    Color barva;
    Point2D.Double body[];
    Point2D.Double transBody[];
    Point2D.Double centroid; // teziste polygonu - budeme tam umistovat text
    Point2D.Double transCentroid;
    // Maxima a minima datove mnoziny
    Point2D.Double max;
    Point2D.Double min;
    // polygon vytvoreny ze zadanych bodu
    Path2D.Double polygon;
    

    public Kraj(String nazev, Point2D.Double body[]) {

        this.barva = Color.WHITE;
        this.nazev = nazev;
        this.body = Arrays.copyOf(body, body.length);

        // spocte bounding box
        zjistiLimity();
        this.transBody = new Point2D.Double[body.length];
        // spocteme teziste
        this.centroid = new Point2D.Double(0, 0);
        for (int i = 0; i < body.length; i++) {
            this.centroid.x += body[i].x;
            this.centroid.y += body[i].y;
        }
        this.centroid.x /= body.length;
        this.centroid.y /= body.length;
        this.transCentroid = new Point2D.Double();

    }

    /**
     * Transformuje souradnice vsech bodu do souradnicoveho systemu
     * vykreslovaciho okna.
     *
     * @param sirka Sirka okna v pixelech, do ktereho se budou data vykreslovat
     * @param vyska Vyska okna v pixelech, do ktereho se budou data vykreslovat
     */
    private void transformujDataDoOkna(int sirka, int vyska) {
        // namerene X, Y hodnoty musime prevest do prostoru okna, ve kterem zobrazujeme
        double convertX = sirka / (max.x - min.x);
        double convertY = vyska / (max.y - min.y);
        // TODO - chceme ale zachovat pomer stran -> vybereme tu, ktera je mensi 
        // tj chceme data radsi mensi v obou rozmerech nez tak velka, ze se v jednom nevejdou do okna
        double convert = convertX > convertY ? convertY : convertX;
        // Prevod z prostoru namerenych hodnot do prostoru zobrazovaciho okna:
        // nejprve stotoznime minimum s 0 (tj. od vsech hodnot odecteme minimum,
        // jednotlive vzorky tak prevedeme na interval <0, nekonecno> a pote prenasobime
        // vyse vypoctenou konstantou.
        // Take je treba prehodit Yove souradnice: "vyska - ...."
        for (int j = 0; j < body.length; j++) {
            double transX = (body[j].x - min.x) * convert;
            double transY = vyska - (body[j].y - min.y) * convert;
            transBody[j] = new Point2D.Double(transX, transY);
        }
        transCentroid.x = (centroid.x - min.x) * convert;
        transCentroid.y = vyska - (centroid.y - min.y) * convert;
        // vytvori instanci tridy path2D z transformovanych bodu
        this.polygon = new Path2D.Double();
        this.polygon.moveTo(transBody[0].x, transBody[0].y);
        for (int i = 1; i < body.length; i++) {
            this.polygon.lineTo(transBody[i].x, transBody[i].y);
        }
        this.polygon.closePath();

    }

    private void vykresliKraje(Graphics2D g2, int sirka, int vyska) {




        if (nazev.equals("JIHOMORAVSKY")) {
            JihomoravskyVykresleni(g2);
        }
        if (nazev.equals("JIHOCESKY")) {
            JihoceskyVykresleni(g2);
        }
        if (nazev.equals("ZLINSKY")) {
            ZlinskyVykresleni(g2);
        }
        if (nazev.equals("OLOMOUCKY")) {
            OlomouckyVykresleni(g2);
        }

        Font font = new Font("Arial", Font.BOLD, 11);
        g2.setFont(font);

        kraje[pocet] = polygon;
        System.out.println(pocet + ": " + kraje[pocet]);
        pocet++;


        TextLayout popis = new TextLayout(nazev, font, g2.getFontRenderContext());
        double vyskaTextu = popis.getAscent() + popis.getDescent();

        Area k = new Area(this.polygon);
        Rectangle2D.Double r = new Rectangle2D.Double(
                transCentroid.x, transCentroid.y - vyskaTextu,
                popis.getBounds().getWidth(), popis.getBounds().getHeight());



        g2.scale(0.6, 1);
        g2.setStroke(new BasicStroke(2));

        g2.setColor(this.barva);
        g2.fill(polygon);

        g2.setColor(Color.black);
        g2.draw(polygon);
        g2.scale(1.5, 1);







        if (nazev.equals("KARLOVARSKY")) {
            popis.draw(g2, -20 + (float) transCentroid.x - ((float) transCentroid.x / 2), (float) transCentroid.y);
        } else if (nazev.equals("STREDOCESKY")) {
            popis.draw(g2, -10 + (float) transCentroid.x - ((float) transCentroid.x / 2), 33 + (float) transCentroid.y);
        } else if (nazev.equals("PLZENSKY")) {
            popis.draw(g2, -20 + (float) transCentroid.x - ((float) transCentroid.x / 2), (float) transCentroid.y);
        } else if (nazev.equals("PRAHA")) {
            popis.draw(g2, -10 + (float) transCentroid.x - ((float) transCentroid.x / 2), (float) transCentroid.y);
        } else if (nazev.equals("LIBERECKY")) {
            popis.draw(g2, -5 + (float) transCentroid.x - ((float) transCentroid.x / 2), -5 + (float) transCentroid.y);
        } else if (nazev.equals("JIHOCESKY")) {
            popis.draw(g2, -15 + (float) transCentroid.x - ((float) transCentroid.x / 2), +20 + (float) transCentroid.y);
        } else if (nazev.equals("VYSOCINA")) {
            popis.draw(g2, +1 + (float) transCentroid.x - ((float) transCentroid.x / 2), -5 + (float) transCentroid.y);
        } else if (nazev.equals("KRALOVEHRADECKY")) {
            popis.draw(g2, -45 + (float) transCentroid.x - ((float) transCentroid.x / 2), -5 + (float) transCentroid.y);
        } else if (nazev.equals("JIHOMORAVSKY")) {
            g2.scale(1, 0.9);
            popis.draw(g2, -15 + (float) transCentroid.x - ((float) transCentroid.x / 2), +35 + (float) transCentroid.y);
        } else if (nazev.equals("OLOMOUCKY")) {
            g2.scale(0.91, 1);
            popis.draw(g2, -5 + (float) transCentroid.x - ((float) transCentroid.x / 2), +15 + (float) transCentroid.y);
        } else if (nazev.equals("ZLINSKY")) {
            g2.scale(0.81, 1);
            popis.draw(g2, 28 + (float) transCentroid.x - ((float) transCentroid.x / 2), +10 + (float) transCentroid.y);
        } else if (nazev.equals("MORAVSKOSLEZSKY")) {
            popis.draw(g2, -18 + (float) transCentroid.x - ((float) transCentroid.x / 2), +10 + (float) transCentroid.y);
        } else {
            popis.draw(g2, (float) transCentroid.x - ((float) transCentroid.x / 2), 5 + (float) transCentroid.y);
        }



    }

    /**
     * Provede vykresleni kraje v ramci zadaneho kontextu
     *
     * @param g2 Graficky kontext pro vykreslovani
     * @param sirkaKontextu Sirka ciloveho platna v pixelech
     * @param vyskaKontextu Vyska ciloveho platna v pixelech
     * @param offsetOkraju Odsazeni od okraje platna
     */
    public void vykresli(Graphics2D g2,
            int sirkaKontextu, int vyskaKontextu, int offsetOkraju, int xpozice, int ypozice) {
        // data je treba transforomovat do souradnicoveho systemu okna
        int sirkaSOdsazenim = sirkaKontextu - 2 * offsetOkraju;
        int vyskaSOdsazenim = vyskaKontextu - 2 * offsetOkraju;
        // tranformace probiha pri kazdem vykreslovani - zamyslete se nad tim, 
        // jak tuto neefektivitu odstranit aniz by se omezila funkcnost aplikace 
        // (reakce na zmenu velikosti okna, reakce pri pridani / odebrani dat aj.) 		
        this.transformujDataDoOkna(sirkaSOdsazenim, vyskaSOdsazenim);
        // veskere vykresleni posuneme o nas offset
        AffineTransform puvodniTrans = g2.getTransform();
        g2.translate(offsetOkraju + xpozice, offsetOkraju + ypozice);

        this.vykresliKraje(g2, sirkaSOdsazenim, vyskaSOdsazenim);

        // vratime zpet puvodni transformaci
        g2.setTransform(puvodniTrans);
    }

    private void zjistiLimity() {

        max = new Point2D.Double(-Double.MAX_VALUE, -Double.MAX_VALUE);
        min = new Point2D.Double(Double.MAX_VALUE, Double.MAX_VALUE);
        for (int i = 0; i < body.length; i++) {
            if (body[i].getX() < min.getX()) {
                min.x = body[i].getX();
            }
            if (body[i].getY() < min.getY()) {
                min.y = body[i].getY();
            }
            if (body[i].getX() > max.getX()) {
                max.x = body[i].getX();
            }
            if (body[i].getY() > max.getY()) {
                max.y = body[i].getY();
            }
        }
    }

    public String getNazev() {
        return nazev;
    }

    public void setNazev(String nazev) {
        this.nazev = nazev;
    }

    public Color getBarva() {
        return barva;
    }

    public void setBarva(Color barva) {
        this.barva = barva;
    }

    public Point2D.Double getMax() {
        return max;
    }

    public void setMax(Point2D.Double max) {
        this.max = max;
    }

    public Point2D.Double getMin() {
        return min;
    }

    public void setMin(Point2D.Double min) {
        this.min = min;
    }

    public Path2D.Double getPolygon() {
        return polygon;
    }

    public void setPolygon(Path2D.Double polygon) {
        this.polygon = polygon;
    }

    public Point2D.Double[] getBody() {
        return body;
    }

    public void setBody(Point2D.Double[] body) {
        this.body = body;
    }

    public Point2D.Double[] getTransBody() {
        return transBody;
    }

    public void setTransBody(Point2D.Double[] transBody) {
        this.transBody = transBody;
    }

    public void JihomoravskyVykresleni(Graphics g) {
        Graphics2D g2 = (Graphics2D) g;
        g2.scale(1, 1.15);
    }

    public void JihoceskyVykresleni(Graphics g) {
        Graphics2D g2 = (Graphics2D) g;
        g2.scale(0.91, 1.11);
    }

    public void ZlinskyVykresleni(Graphics g) {
        Graphics2D g2 = (Graphics2D) g;
        g2.scale(1.2, 1);
    }

    public void OlomouckyVykresleni(Graphics g) {
        Graphics2D g2 = (Graphics2D) g;
        g2.scale(1.11, 1);
    }

    public Shape[] getShapes() {
        return kraje;
    }
}
